@@ -65,23 +65,27 @@ Mail::DKIM::Signer - generates a DKIM signature for a message
   # create a signer using a custom policy
   my $dkim = Mail::DKIM::Signer->new(
                   Policy => $policyfn,
-                  KeyFile => "private.key",
              );
 
-You must always specify the name of a private key file. In addition,
-you must specify a policy object, or specify the algorithm, method,
-domain, and selector to use. Use of the policy object lets you defer
-the determination of algorithm, method, domain and selector until
-the message being signed has been partially read.
+The "default policy" is to create a DKIM signature using the specified
+parameters, but only if the message's sender matches the domain.
+The following parameters can be passed to this new() method to
+influence the resulting signature:
+Algorithm, Method, Domain, Selector, KeyFile, Identity, Timestamp.
 
-See Mail::DKIM::SignerPolicy for more information about policy objects.
+If you want different behavior, you can provide a "signer policy"
+instead. A signer policy is a subroutine or class that determines
+signature parameters after the message's headers have been parsed.
+See the section L</"SIGNER POLICIES"> below for more information.
+
+See L<Mail::DKIM::SignerPolicy> for more information about policy objects.
 
 =cut
 
 package Mail::DKIM::Signer;
 use base "Mail::DKIM::Common";
 use Carp;
-our $VERSION = '0.29';
+our $VERSION = '0.30';
 
 # PROPERTIES
 #
@@ -107,15 +111,15 @@ our $VERSION = '0.29';
 # $dkim->{Selector}
 #   identifies name of the selector identifying the key
 #
+# $dkim->{Key}
+#   the loaded private key
+#
 # private:
 #
 # $dkim->{algorithms} = []
 #   an array of algorithm objects... an algorithm object is created for
 #   each signature being added to the message
 #
-# $dkim->{private}
-#   the loaded private key
-#
 # $dkim->{result}
 #   result of the signing policy: "signed" or "skipped"
 #
@@ -130,11 +134,9 @@ sub init
 
 	if (defined $self->{KeyFile})
 	{
-		$self->{private} = Mail::DKIM::PrivateKey->load(
+		$self->{Key} ||= Mail::DKIM::PrivateKey->load(
 				File => $self->{KeyFile});
 	}
-	croak "No private key specified"
-		unless ($self->{private});
 	
 	unless ($self->{"Algorithm"})
 	{
@@ -210,14 +212,18 @@ sub finish_header
 		}
 
 		$self->add_signature(
-			new Mail::DKIM::Signature(
+			Mail::DKIM::Signature->new(
 				Algorithm => $self->{"Algorithm"},
 				Method => $self->{"Method"},
 				Headers => $self->headers,
 				Domain => $self->{"Domain"},
 				Selector => $self->{"Selector"},
+				Key => $self->{"Key"},
+				KeyFile => $self->{"KeyFile"},
 				($self->{"Identity"} ?
 					(Identity => $self->{"Identity"}) : ()),
+				($self->{"Timestamp"} ?
+					(Timestamp => $self->{"Timestamp"}) : ()),
 			));
 	}
 
@@ -241,14 +247,28 @@ sub finish_body
 		# finished canonicalizing
 		$algorithm->finish_body;
 
+		# load the private key file if necessary
+		my $signature = $algorithm->signature;
+		my $key = $signature->{Key}
+			|| $signature->{KeyFile}
+			|| $self->{Key}
+			|| $self->{KeyFile};
+		if (not ref $key)
+		{
+			$key = Mail::DKIM::PrivateKey->load(
+					File => $key);
+		}
+		$key
+			or die "no key available to sign with\n";
+
 		# compute signature value
-		my $signb64 = $algorithm->sign($self->{private});
-		$algorithm->signature->data($signb64);
+		my $signb64 = $algorithm->sign($key);
+		$signature->data($signb64);
 
 		# insert linebreaks in signature data, if desired
-		$algorithm->signature->prettify_safe();
+		$signature->prettify_safe();
 
-		$self->{signature} = $algorithm->signature;
+		$self->{signature} = $signature;
 		$self->{result} = "signed";
 	}
 }
@@ -276,7 +296,7 @@ and generates a signature.
 
 Signer policies can use this method to specify complete parameters for
 the signature to add, including what type of signature. For more information,
-see Mail::DKIM::SignerPolicy.
+see L<Mail::DKIM::SignerPolicy>.
 
 =cut
 
@@ -287,7 +307,7 @@ sub add_signature
 
 	# create a canonicalization filter and algorithm
 	my $algorithm_class = $signature->get_algorithm_class(
-			$self->{"Algorithm"});
+			$signature->algorithm);
 	my $algorithm = $algorithm_class->new(
 			Signature => $signature,
 			Debug_Canonicalization => $self->{Debug_Canonicalization},
@@ -394,6 +414,44 @@ sub want_header
 	return scalar grep { lc($_) eq lc($header_name) } @DEFAULT_HEADERS;
 }
 
+=head2 key() - get or set the private key object
+
+  my $key = $dkim->key;
+
+  $dkim->key(Mail::DKIM::PrivateKey->load(File => "private.key"));
+
+=cut
+
+sub key
+{
+	my $self = shift;
+	if (@_)
+	{
+		$self->{Key} = shift;
+		$self->{KeyFile} = undef;
+	}
+	return $self->{Key};
+}
+
+=head2 key_file() - get or set the filename containing the private key
+
+  my $filename = $dkim->key_file;
+
+  $dkim->key_file("private.key");
+
+=cut
+
+sub key_file
+{
+	my $self = shift;
+	if (@_)
+	{
+		$self->{Key} = undef;
+		$self->{KeyFile} = shift;
+	}
+	return $self->{KeyFile};
+}
+
 =head2 method() - get or set the selected canonicalization method
 
   $alg = $dkim->method;
@@ -418,7 +476,7 @@ sub method
 
 Returns the "originator address" found in the message. This is typically
 the (first) name and email address found in the From: header. The returned
-object is of type Mail::Address. To get just the email address part, do:
+object is of type L<Mail::Address>. To get just the email address part, do:
 
   my $email = $dkim->message_originator->address;
 
@@ -535,7 +593,7 @@ specified in the new() constructor.
 
 =head1 SEE ALSO
 
-Mail::DKIM::SignerPolicy
+L<Mail::DKIM::SignerPolicy>
 
 =head1 AUTHOR
 
